import path from 'path';
import process from 'process';
import { defineConfig, loadEnv } from 'vite';
import vue from '@vitejs/plugin-vue';
import AutoImport from 'unplugin-auto-import/vite';
import Components from 'unplugin-vue-components/vite';
import { ElementPlusResolver, ArcoResolver, VantResolver } from 'unplugin-vue-components/resolvers';
import { visualizer } from 'rollup-plugin-visualizer';
import vueI18n from '@intlify/vite-plugin-vue-i18n';
import { chunkSplitPlugin } from 'vite-plugin-chunk-split';
import imagemin from 'unplugin-imagemin/vite';
import DefineOptions from 'unplugin-vue-define-options/vite';
import { viteZip } from 'vite-plugin-zip-file';
import fs from 'fs-extra';

import { cliDir, appDir, srcDir, yicodeDir, cacheDir } from './config.js';
import { fnGetFileProtocolPath, fnImportModule } from './utils.js';

// TODO: 完成图标使用方案

export default defineConfig(async ({ command, mode }) => {
    let { yiviteConfig } = await fnImportModule(fnGetFileProtocolPath(path.resolve(yicodeDir, 'yivite.config.mjs')), {});
    let { dependencies = {} } = fs.readJsonSync(path.resolve(appDir, 'package.json'), { throws: false }) || {};

    // 每个项目依赖包进行分割
    let splitDependencies = {};
    for (let prop in dependencies) {
        if (dependencies.hasOwnProperty(prop)) {
            splitDependencies[prop] = [prop];
        }
    }

    return Object.assign(
        {
            plugins: [
                //
                vue({
                    reactivityTransform: true
                }),
                AutoImport({
                    include: [/\.[tj]sx?$/, /\.vue$/, /\.vue\?vue/, /\.md$/],
                    imports: ['vue'],
                    dirs: [
                        //
                        path.resolve(srcDir, 'components'),
                        path.resolve(srcDir, 'plugins')
                    ],
                    vueTemplate: true,
                    dts: '.yicode/auto-imports.d.ts',
                    resolvers: [ElementPlusResolver(), ArcoResolver()]
                }),
                Components({
                    directoryAsNamespace: true,
                    resolvers: [
                        //
                        ElementPlusResolver(),
                        ArcoResolver({
                            sideEffect: true
                        }),
                        VantResolver()
                    ]
                }),
                vueI18n({
                    include: path.resolve(srcDir, 'i18n/**')
                }),
                chunkSplitPlugin({
                    strategy: 'default',
                    customSplitting: Object.assign(splitDependencies, yiviteConfig?.viteConfig?.chunkSplit || {})
                }),
                DefineOptions(),
                imagemin({
                    mode: 'sharp',
                    compress: {
                        jpg: {
                            quality: 80
                        },
                        jpeg: {
                            quality: 80
                        },
                        png: {
                            quality: 80
                        },
                        webp: {
                            quality: 80
                        }
                    },
                    cache: false
                }),
                viteZip({
                    folderPath: path.resolve(appDir, 'dist'),
                    outPath: cacheDir,
                    zipName: yiviteConfig?.viteConfig?.viteZip?.zipName || 'dist.zip',
                    enabled: mode === 'production' ? true : false
                }),
                visualizer({
                    open: false,
                    brotliSize: true,
                    filename: '.cache/buildReport.html'
                })
            ],
            css: {
                preprocessorOptions: {
                    scss: {
                        additionalData: `@use "@/styles/variable.scss" as *;`
                    }
                }
            },
            resolve: {
                alias: [
                    {
                        find: '@',
                        replacement: path.resolve(srcDir)
                    }
                ]
            },
            root: appDir,
            base: './',
            envDir: path.resolve(srcDir, 'env'),
            build: {
                rollupOptions: {
                    plugins: []
                }
            },
            server: {
                host: '127.0.0.1'
            }
        },
        yiviteConfig?.config || {}
    );
});
